@@ -42,9 +42,9 @@ and installing updates. Please free some space.' % (mounts[mount], free_mb)
     # important glib errors/assertions (which should not have private data)
     if 'ExecutablePath' in report:
         path = report['ExecutablePath']
-        gtk_like = (apport.hookutils.links_with_shared_library(path, 'libgtk') or
-                    apport.hookutils.links_with_shared_library(path, 'libgtk-3') or
-                    apport.hookutils.links_with_shared_library(path, 'libX11'))
+        gtk_like = (apport.fileutils.links_with_shared_library(path, 'libgtk') or
+                    apport.fileutils.links_with_shared_library(path, 'libgtk-3') or
+                    apport.fileutils.links_with_shared_library(path, 'libX11'))
         if gtk_like and apport.hookutils.in_session_of_problem(report):
             xsession_errors = apport.hookutils.xsession_errors()
             if xsession_errors:
@@ -56,7 +56,7 @@ and installing updates. Please free some space.' % (mounts[mount], free_mb)
         for lib in re.finditer(r'\s(/[^ ]+\.so[.0-9]*)$', report['ProcMaps'], re.M):
             if not apport.fileutils.likely_packaged(lib.group(1)):
                 local_libs.add(lib.group(1))
-        if local_libs:
+        if ui and local_libs:
             if not ui.yesno('''The crashed program seems to use third-party or local libraries:
 
 %s
