@@ -15,10 +15,10 @@ test_executable = '/usr/bin/yes'
 test_package = 'coreutils'
 test_source = 'coreutils'
 
-# (core ulimit (kb), expect core signal, expect core file, expect report)
+# (core ulimit (bytes), expect core signal, expect core file, expect report)
 core_ulimit_table = [(1, False, False, False),
-                     (10, True, False, True),
-                     (10000, True, True, True),
+                     (1000, True, False, True),
+                     (1000000, True, True, True),
                      (-1, True, True, True)]
 
 required_fields = ['ProblemType', 'CoreDump', 'Date', 'ExecutablePath',
@@ -314,7 +314,7 @@ class T(unittest.TestCase):
             apport.fileutils.delete_report(self.test_report)
 
         # for SIGQUIT we only expect core files, no report
-        resource.setrlimit(resource.RLIMIT_CORE, (10000, -1))
+        resource.setrlimit(resource.RLIMIT_CORE, (1000000, -1))
         self.do_crash(expect_corefile=True, sig=signal.SIGQUIT)
         self.assertEqual(apport.fileutils.get_all_reports(), [])
 
@@ -622,7 +622,19 @@ class T(unittest.TestCase):
         if sleep > 0:
             time.sleep(sleep)
         os.kill(pid, sig)
-        result = os.waitpid(pid, 0)[1]
+        # wait max 5 seconds for the process to die
+        timeout = 50
+        while timeout >= 0:
+            (p, result) = os.waitpid(pid, os.WNOHANG)
+            if p != 0:
+                break
+            time.sleep(0.1)
+            timeout -= 1
+        else:
+            os.kill(pid, signal.SIGKILL)
+            os.waitpid(pid, 0)
+            self.fail('test process does not die on signal %i' % sig)
+
         self.assertFalse(os.WIFEXITED(result), 'test process did not exit normally')
         self.assertTrue(os.WIFSIGNALED(result), 'test process died due to signal')
         self.assertEqual(os.WCOREDUMP(result), expect_coredump)
@@ -693,6 +705,9 @@ class T(unittest.TestCase):
 # main
 #
 
+# ensure we don't inherit an ignored SIGQUIT
+signal.signal(signal.SIGQUIT, signal.SIG_DFL)
+
 with open('/proc/sys/kernel/core_pattern') as f:
     core_pattern = f.read().strip()
 if core_pattern[0] != '|':
